@@ -1,6 +1,7 @@
 import argparse, sys
 
 import scripts
+from xklb.dl_extract import dl_add, dl_block, dl_download, dl_update
 from xklb.fs_actions import filesystem, listen, read, view, watch
 from xklb.fs_extract import main as fs_add
 from xklb.playback import playback_next, playback_now, playback_pause, playback_stop
@@ -12,7 +13,7 @@ from xklb.tube_extract import tube_add, tube_update
 from xklb.utils import SC, log
 
 
-def lb_usage():
+def usage() -> str:
     return """xk media library [lb]
 
     local media subcommands:
@@ -25,14 +26,21 @@ def lb_usage():
       filesystem [fs]              Browse files
       bigdirs [largefolders]       Discover folders which take up much room
       dedupe                       Deduplicate audio files
+      christen                     Cleanse files by giving them a new name
 
     online media subcommands:
       tubeadd [ta, xt]             Create a tube database; Add playlists
-      tubeupdate [tu]              Get new videos from saved playlists
+      tubeupdate [tu]              Add new videos from saved playlists
       tubelist [playlists]         List added playlists
       tubewatch [tw, entries]      Watch the tube
       tubelisten [tl]              Listen to the tube
 
+    download subcommands (largely untested, under construction):
+      dladd [da]                   Create a download database; Add URLs
+      dlupdate [du]                Add new videos from saved playlists
+      download [dl]                Download media
+      block                        Prevent downloading from specific channels
+
     playback subcommands:
       now                          Print what is currently playing
       next                         Play next file
@@ -45,12 +53,12 @@ def lb_usage():
     """
 
 
-def print_help(parser):
-    print(lb_usage())
+def print_help(parser) -> None:
+    print(usage())
     print(parser.epilog)
 
 
-def lb(args=None):
+def lb(args=None) -> None:
     if args:
         sys.argv[2:] = args
 
@@ -83,6 +91,9 @@ def lb(args=None):
     subp_dedupe = subparsers.add_parser("dedupe", add_help=False)
     subp_dedupe.set_defaults(func=scripts.deduplicate_music)
 
+    subp_christen = subparsers.add_parser("christen", add_help=False)
+    subp_christen.set_defaults(func=scripts.rename_invalid_files)
+
     subp_tubelist = subparsers.add_parser("tubelist", aliases=["playlist", "playlists"], add_help=False)
     subp_tubelist.set_defaults(func=tube_list)
     subp_tubeadd = subparsers.add_parser("tubeadd", aliases=["ta", "xt"], add_help=False)
@@ -109,6 +120,15 @@ def lb(args=None):
     subp_tabs = subparsers.add_parser("tabs", aliases=["tabswatch", "tb"], add_help=False)
     subp_tabs.set_defaults(func=tabs)
 
+    subp_dladd = subparsers.add_parser("dladd", aliases=["da"], add_help=False)
+    subp_dladd.set_defaults(func=dl_add)
+    subp_dlupdate = subparsers.add_parser("dlupdate", aliases=["du"], add_help=False)
+    subp_dlupdate.set_defaults(func=dl_update)
+    subp_download = subparsers.add_parser("download", aliases=["dl"], add_help=False)
+    subp_download.set_defaults(func=dl_download)
+    subp_block = subparsers.add_parser("block", add_help=False)
+    subp_block.set_defaults(func=dl_block)
+
     parser.add_argument("--version", "-V", action="store_true")
     args, _unk = parser.parse_known_args(args)
     if args.version:
@@ -118,7 +138,7 @@ def lb(args=None):
 
     log.info(sys.argv)
     original_argv = sys.argv
-    if len(sys.argv) > 1:
+    if len(sys.argv) >= 2:
         del sys.argv[1]
 
     if hasattr(args, "func"):
@@ -132,7 +152,7 @@ def lb(args=None):
         print_help(parser)
 
 
-def main():
+def main() -> None:
     lb()
 
 
