@@ -1,4 +1,4 @@
-import csv, math, operator, os, platform, re, shutil, socket, subprocess
+import csv, operator, os, platform, re, shutil, socket, subprocess
 from copy import deepcopy
 from io import StringIO
 from pathlib import Path
@@ -7,14 +7,13 @@ from random import randrange
 from shlex import join, quote
 from shutil import which
 from time import sleep
-from typing import List, Union
+from typing import List, Tuple, Union
 
-import ffmpeg, sqlite_utils
+import screeninfo
 from rich.prompt import Confirm
-from screeninfo import get_monitors
 from tabulate import tabulate
 
-from xklb import paths, utils
+from xklb import gui, paths, utils
 from xklb.utils import (
     DEFAULT_MULTIPLE_PLAYBACK,
     SC,
@@ -27,7 +26,7 @@ from xklb.utils import (
 )
 
 
-def generic_player(args):
+def generic_player(args) -> List[str]:
     if platform.system() == "Linux":
         player = ["xdg-open"]
     elif any([p in platform.system() for p in ["Windows", "_NT-", "MSYS"]]):
@@ -41,7 +40,7 @@ def generic_player(args):
     return player
 
 
-def calculate_duration(args, m):
+def calculate_duration(args, m) -> Tuple[int, int]:
     start = 0
     end = m["duration"]
 
@@ -61,7 +60,7 @@ def calculate_duration(args, m):
     return start, end
 
 
-def find_xdg_application(media_file):
+def find_xdg_application(media_file) -> Union[str, None]:
     mimetype = cmd("xdg-mime", "query", "filetype", media_file).stdout
     default_application = cmd("xdg-mime", "query", "default", mimetype).stdout
     player_path = which(default_application.replace(".desktop", ""))
@@ -125,7 +124,7 @@ def parse(args, m=None, media_file=None) -> List[str]:
     return player
 
 
-def mv_to_keep_folder(args, media_file: str):
+def mv_to_keep_folder(args, media_file: str) -> None:
     keep_path = Path(args.keep_dir)
     if not keep_path.is_absolute():
         kp = re.match(args.shallow_organize + "(.*?)/", media_file)
@@ -142,7 +141,7 @@ def mv_to_keep_folder(args, media_file: str):
         args.db.conn.execute("UPDATE media set path = ? where path = ?", [new_path, media_file])
 
 
-def moved_media(args, moved_files: Union[str, list], base_from, base_to):
+def moved_media(args, moved_files: Union[str, list], base_from, base_to) -> int:
     moved_files = utils.conform(moved_files)
     modified_row_count = 0
     if len(moved_files) > 0:
@@ -162,7 +161,7 @@ def moved_media(args, moved_files: Union[str, list], base_from, base_to):
     return modified_row_count
 
 
-def mark_media_watched(args, files):
+def mark_media_watched(args, files) -> int:
     files = utils.conform(files)
     modified_row_count = 0
     if len(files) > 0:
@@ -183,7 +182,7 @@ def mark_media_watched(args, files):
     return modified_row_count
 
 
-def mark_media_deleted(args, paths):
+def mark_media_deleted(args, paths) -> int:
     paths = utils.conform(paths)
 
     modified_row_count = 0
@@ -204,16 +203,18 @@ def mark_media_deleted(args, paths):
     return modified_row_count
 
 
-def delete_media(args, media_file: str):
-    if len(args.prefix) > 0:
-        Path(media_file).unlink()
-    else:
-        utils.trash(media_file)
+def delete_media(args, paths) -> int:
+    paths = utils.conform(paths)
+    for p in paths:
+        if len(args.prefix) > 0:
+            Path(p).unlink(missing_ok=True)
+        else:
+            utils.trash(p)
 
-    mark_media_deleted(args, media_file)
+    return mark_media_deleted(args, paths)
 
 
-def delete_playlists(args, playlists):
+def delete_playlists(args, playlists) -> None:
     with args.db.conn:
         args.db.conn.execute(
             "delete from media where playlist_path in (" + ",".join(["?"] * len(playlists)) + ")", (*playlists,)
@@ -223,61 +224,63 @@ def delete_playlists(args, playlists):
         )
 
 
-def post_act(args, media_file: str):
+def post_act(args, media_file: str, action=None) -> None:
+    action = action or args.post_action
+
     mark_media_watched(args, media_file)
-    if args.post_action == "keep":
+    if action == "keep":
         return
 
     if args.action in [SC.tubelisten, SC.tubewatch]:
-        if args.post_action == "remove":
+        if action == "remove":
             mark_media_deleted(args, media_file)
-        elif args.post_action == "ask":
+        elif action == "ask":
             if not Confirm.ask("Keep?", default=False):
                 mark_media_deleted(args, media_file)
         else:
-            raise Exception("Unrecognized post_action", args.post_action)
+            raise Exception("Unrecognized action", action)
 
     if args.action in [SC.listen, SC.watch]:
-        if args.post_action == "softdelete":
+        if action == "softdelete":
             mark_media_deleted(args, media_file)
 
-        elif args.post_action == "delete":
+        elif action == "delete":
             delete_media(args, media_file)
 
-        elif args.post_action == "delete-if-audiobook":
+        elif action == "delete-if-audiobook":
             if "audiobook" in media_file.lower():
                 delete_media(args, media_file)
 
-        elif args.post_action == "ask":
+        elif action == "ask":
             if not Confirm.ask("Keep?", default=False):
                 delete_media(args, media_file)
 
-        elif args.post_action == "askkeep":
+        elif action == "askkeep":
             if not Confirm.ask("Keep?", default=False):
                 delete_media(args, media_file)
             else:
                 mv_to_keep_folder(args, media_file)
 
         else:
-            raise Exception("Unrecognized post_action", args.post_action)
+            raise Exception("Unrecognized action", action)
 
 
-def override_sort(string):
+def override_sort(sort_expression: str) -> str:
     YEAR_MONTH = lambda var: f"cast(strftime('%Y%m', datetime({var}, 'unixepoch')) as int)"
 
     return (
-        string.replace("month_created", YEAR_MONTH("time_created"))
+        sort_expression.replace("month_created", YEAR_MONTH("time_created"))
         .replace("month_modified", YEAR_MONTH("time_modified"))
         .replace("random", "random()")
         .replace("priority", " play_count, ntile(1000) over (order by size/duration) desc")
     )
 
 
-def get_ordinal_media(args, path):
+def get_ordinal_media(args, path: str) -> str:
     # TODO: maybe try https://dba.stackexchange.com/questions/43415/algorithm-for-finding-the-longest-prefix
 
     similar_videos = []
-    candidate = str(path)
+    candidate = deepcopy(path)
 
     total_media = args.db.execute("select count(*) val from media").fetchone()[0]
     while len(similar_videos) < 2:
@@ -300,12 +303,15 @@ def get_ordinal_media(args, path):
             WHERE 1=1
                 and path like :candidate
                 {'and is_deleted=0' if args.action in [SC.listen, SC.watch] else ''}
-                {'' if (args.play_in_order > 2) else (args.sql_filter or '')}
+                {'' if (args.play_in_order >= 3) else (args.sql_filter or '')}
             ORDER BY path
             LIMIT 1000
             """
         bindings = {"candidate": "%" + candidate + "%"}
-        if not args.play_in_order > 2:
+        if args.play_in_order == 2:
+            if args.include or args.exclude:
+                bindings = {**bindings, "query": args.sql_filter_bindings["query"]}
+        else:
             bindings = {**bindings, **args.sql_filter_bindings}
 
         similar_videos = [d["path"] for d in args.db.query(query, bindings)]
@@ -323,7 +329,7 @@ def get_ordinal_media(args, path):
     return similar_videos[0]
 
 
-def watch_chromecast(args, m, subtitles_file=None):
+def watch_chromecast(args, m: dict, subtitles_file=None) -> Union[subprocess.CompletedProcess, None]:
     if "vlc" in args.player:
         catt_log = cmd(
             "vlc",
@@ -354,7 +360,7 @@ def watch_chromecast(args, m, subtitles_file=None):
     return catt_log
 
 
-def listen_chromecast(args, m):
+def listen_chromecast(args, m: dict) -> Union[subprocess.CompletedProcess, None]:
     Path(paths.CAST_NOW_PLAYING).write_text(m["path"])
     Path(paths.FAKE_SUBTITLE).touch()
     if args.with_local:
@@ -375,7 +381,7 @@ def listen_chromecast(args, m):
     return catt_log
 
 
-def socket_play(args, m):
+def socket_play(args, m: dict) -> None:
     if args.sock is None:
         subprocess.Popen(["mpv", "--idle", "--input-ipc-server=" + args.mpv_socket])
         while not os.path.exists(args.mpv_socket):
@@ -407,11 +413,11 @@ def socket_play(args, m):
     sleep(args.interdimensional_cable)
 
 
-def geom(x_size, y_size, x, y):
+def geom(x_size, y_size, x, y) -> List[str]:
     return [f"--geometry={x_size}%x{y_size}%+{x}%+{y}%"]
 
 
-def geom_walk(v=1, h=1):
+def geom_walk(v=1, h=1) -> List[List[str]]:
     va = 100 // v
     ha = 100 // h
 
@@ -425,91 +431,26 @@ def geom_walk(v=1, h=1):
     return geoms
 
 
-def vstack(display, qty):
-    mapper = {
-        1: [["--fs", f'--fs-screen-name="{display.name}"']],
-        2: geom_walk(v=2, h=1),
-        4: geom_walk(v=2, h=2),
-        6: geom_walk(v=2, h=3),
-        8: geom_walk(v=2, h=4),
-        9: geom_walk(v=3, h=3),
-        10: geom_walk(v=2, h=5),
-        12: geom_walk(v=3, h=4),
-        14: geom_walk(v=2, h=7),
-        15: geom_walk(v=3, h=5),
-        16: geom_walk(v=4, h=4),
-        18: geom_walk(v=3, h=6),
-        20: geom_walk(v=4, h=5),
-        21: geom_walk(v=3, h=7),
-        22: geom_walk(v=2, h=11),
-        24: geom_walk(v=4, h=6),
-        26: geom_walk(v=2, h=13),
-        28: geom_walk(v=4, h=7),
-        30: geom_walk(v=5, h=6),
-        32: geom_walk(v=4, h=8),
-        36: geom_walk(v=6, h=6),
-        49: geom_walk(v=7, h=7),
-        64: geom_walk(v=8, h=8),
-        81: geom_walk(v=9, h=9),
-        100: geom_walk(v=10, h=10),
-    }
-    if qty in mapper.keys():
-        holes = mapper[qty]
+def grid_stack(display, qty, swap=False) -> List[List[str]]:
+    if qty == 1:
+        return [[f'--screen-name="{display.name}"', "--fs", f'--fs-screen-name="{display.name}"']]
     else:
         dv = list(utils.divisor_gen(qty))
         if not dv:
-            holes = geom_walk(v=qty)
+            vh = (qty, 1)
         else:
             v = dv[len(dv) // 2]
             h = qty // v
-            holes = geom_walk(v=v, h=h)
+            vh = (v, h)
 
+    v, h = vh
+    if swap:
+        h, v = v, h
+    holes = geom_walk(v=v, h=h)
     return [[f'--screen-name="{display.name}"', *hole] for hole in holes]
 
 
-def hstack(display, qty):
-    mapper = {
-        1: [["--fs", f'--fs-screen-name="{display.name}"']],
-        2: geom_walk(v=1, h=2),
-        4: geom_walk(v=2, h=2),
-        6: geom_walk(v=3, h=2),
-        8: geom_walk(v=4, h=2),
-        9: geom_walk(v=3, h=3),
-        10: geom_walk(v=5, h=2),
-        12: geom_walk(v=4, h=3),
-        14: geom_walk(v=7, h=2),
-        15: geom_walk(v=5, h=3),
-        16: geom_walk(v=4, h=4),
-        18: geom_walk(v=6, h=3),
-        20: geom_walk(v=5, h=4),
-        21: geom_walk(v=7, h=3),
-        22: geom_walk(v=11, h=2),
-        24: geom_walk(v=6, h=4),
-        26: geom_walk(v=13, h=2),
-        28: geom_walk(v=7, h=4),
-        30: geom_walk(v=6, h=5),
-        32: geom_walk(v=8, h=4),
-        36: geom_walk(v=6, h=6),
-        49: geom_walk(v=7, h=7),
-        64: geom_walk(v=8, h=8),
-        81: geom_walk(v=9, h=9),
-        100: geom_walk(v=10, h=10),
-    }
-    if qty in mapper.keys():
-        holes = mapper[qty]
-    else:
-        dv = list(utils.divisor_gen(qty))
-        if not dv:
-            holes = geom_walk(h=qty)
-        else:
-            h = dv[len(dv) // 2]
-            v = qty // h
-            holes = geom_walk(v=v, h=h)
-
-    return [[f'--screen-name="{display.name}"', *hole] for hole in holes]
-
-
-def get_display_by_name(displays, screen_name):
+def get_display_by_name(displays, screen_name) -> List[screeninfo.Monitor]:
     for d in displays:
         if d.name == screen_name:
             return [d]
@@ -518,25 +459,25 @@ def get_display_by_name(displays, screen_name):
     raise Exception(f'Display "{screen_name}" not found. I see: "{display_names}"')
 
 
-def which_stack(args, display):
+def is_hstack(args, display) -> bool:
     if args.hstack or args.portrait:
-        return hstack
+        return True
     elif args.vstack:
-        return vstack
-    elif display.width > display.height:
-        return vstack
+        return False
+    elif display.width > display.height:  # wide
+        return False
     else:  # tall or square: prefer horizontal split
-        return hstack
+        return True
 
 
-def get_multiple_player_template(args):
-    displays = get_monitors()
+def get_multiple_player_template(args) -> List[str]:
+    displays = screeninfo.get_monitors()
     if args.screen_name:
         displays = get_display_by_name(displays, args.screen_name)
 
     if args.multiple_playback == DEFAULT_MULTIPLE_PLAYBACK and len(displays) == 1:
         args.multiple_playback = 2
-    elif args.multiple_playback == DEFAULT_MULTIPLE_PLAYBACK and len(displays) > 1:
+    elif args.multiple_playback == DEFAULT_MULTIPLE_PLAYBACK and len(displays) >= 2:
         args.multiple_playback = len(displays)
     elif args.multiple_playback < len(displays):
         # play videos on supporting screens but not active one
@@ -552,15 +493,14 @@ def get_multiple_player_template(args):
         if remainder > 0 and (d_idx + 1) == len(displays):
             qty += remainder
 
-        fn_stack = which_stack(args, display)
-        players.extend(fn_stack(display, qty))
+        players.extend(grid_stack(display, qty, swap=is_hstack(args, display)))
 
     log.debug(players)
 
     return players
 
 
-def multiple_player(args, media):
+def multiple_player(args, media) -> None:
     def open_player(template_args, m):
         print(m["path"])
         mp_args = ["--window-scale=1", "--no-border", "--no-keepaspect-window"]
@@ -591,7 +531,18 @@ def multiple_player(args, media):
                                 exit(r.returncode)
 
                         if args.action in [SC.listen, SC.watch, SC.tubelisten, SC.tubewatch]:
-                            post_act(args, m["path"])
+                            if args.post_action in ["ask", "askkeep"]:
+                                r = gui.askkeep(m["path"], len(media))
+                                if r == "DELETE":
+                                    post_act(args, m["path"], action="delete")
+                                elif r == "KEEP" and args.post_action == "ask":
+                                    post_act(args, m["path"], action="keep")
+                                elif r == "KEEP" and args.post_action == "askkeep":
+                                    mv_to_keep_folder(args, m["path"])
+                                else:
+                                    raise Exception("I did not quite catch that... what did you say?")
+                            else:
+                                post_act(args, m["path"])
 
                         m = media.pop()
                         players[t_idx] = {**m, "process": open_player(t, m)}
@@ -603,15 +554,11 @@ def multiple_player(args, media):
             m["process"].kill()
 
 
-def local_player(args, m, media_file):
+def local_player(args, m, media_file) -> subprocess.CompletedProcess:
     if system() == "Windows" or args.action in [SC.watch, SC.tubewatch]:
         r = cmd(*args.player, media_file, strict=False)
     else:  # args.action in [SC.listen, SC.tubelisten]
         r = cmd_interactive(*args.player, media_file, strict=False)
-    if r.returncode != 0:
-        print("Player exited with code", r.returncode)
-        if not args.ignore_errors:
-            exit(r.returncode)
 
     if args.player_need_sleep:
         if hasattr(m, "duration"):
@@ -620,8 +567,10 @@ def local_player(args, m, media_file):
             delay = 10  # TODO: idk
         sleep(delay)
 
+    return r
 
-def printer(args, query, bindings):
+
+def printer(args, query, bindings) -> None:
     if "a" in args.print:
         query = f"""select
             "Aggregate" as path
@@ -639,7 +588,7 @@ def printer(args, query, bindings):
     if "v" in args.print and Path(args.watch_later_directory).exists():
         db_resp = utils.mpv_enrich2(args, db_resp)
 
-    if args.verbose > 1 and args.cols and "*" in args.cols:
+    if args.verbose >= 2 and args.cols and "*" in args.cols:
         breakpoint()
 
     if len(db_resp) == 0:
@@ -703,7 +652,7 @@ def printer(args, query, bindings):
         print(tabulate(tbl, tablefmt="fancy_grid", headers="keys", showindex=False))  # type: ignore
 
         if args.action in [SC.listen, SC.watch, SC.tubelisten, SC.tubewatch]:
-            if len(db_resp) > 1:
+            if len(db_resp) >= 2:
                 print(f"{len(db_resp)} media" + (f" (limited to {args.limit})" if args.limit else ""))
 
             duration = sum(map(lambda m: m.get("duration") or 0, db_resp))
