@@ -1,68 +1,13 @@
 import argparse, operator
 from copy import deepcopy
+from typing import Tuple
 
 from tabulate import tabulate
 
 from xklb import db, utils
-from xklb.fs_actions import construct_search_bindings, parse_args, process_playqueue
+from xklb.play_actions import construct_search_bindings, parse_args, process_playqueue
 from xklb.player import delete_playlists
-from xklb.utils import DEFAULT_PLAY_QUEUE, SC, dict_filter_bool, human_time, log
-
-# TODO: add cookiesfrombrowser: ('firefox', ) as a default
-# cookiesfrombrowser: ('vivaldi', ) # should not crash if not installed ?
-
-default_ydl_opts = {
-    "extract_flat": True,
-    "lazy_playlist": True,
-    "skip_download": True,
-    "check_formats": False,
-    "no_check_certificate": True,
-    "no_warnings": True,
-    "ignore_no_formats_error": True,
-    "skip_playlist_after_errors": 20,
-    "quiet": True,
-    "dynamic_mpd": False,
-    "youtube_include_dash_manifest": False,
-    "youtube_include_hls_manifest": False,
-    "clean_infojson": False,
-    "playlistend": 20000,
-    "rejecttitle": "|".join(
-        [
-            "Trailer",
-            "Sneak Peek",
-            "Preview",
-            "Teaser",
-            "Promo",
-            "Crypto",
-            "Montage",
-            "Bitcoin",
-            "Apology",
-            " Clip",
-            "Clip ",
-            "Best of",
-            "Compilation",
-            "Top 10",
-            "Top 9",
-            "Top 8",
-            "Top 7",
-            "Top 6",
-            "Top 5",
-            "Top 4",
-            "Top 3",
-            "Top 2",
-            "Top Ten",
-            "Top Nine",
-            "Top Eight",
-            "Top Seven",
-            "Top Six",
-            "Top Five",
-            "Top Four",
-            "Top Three",
-            "Top Two",
-        ]
-    ),
-}
-
+from xklb.utils import DEFAULT_PLAY_QUEUE, SC, human_time, log
 
 tube_include_string = (
     lambda x: f"""and (
@@ -81,7 +26,7 @@ tube_exclude_string = (
 )
 
 
-def construct_tube_query(args):
+def construct_tube_query(args) -> Tuple[str, dict]:
     cf = []
     bindings = {}
 
@@ -131,29 +76,33 @@ def construct_tube_query(args):
     return query, bindings
 
 
-def tube_watch():
+def tube_watch() -> None:
     args = parse_args(SC.tubewatch, "tube.db", default_chromecast="Living Room TV")
     process_playqueue(args, construct_tube_query)
 
 
-def tube_listen():
+def tube_listen() -> None:
     args = parse_args(SC.tubelisten, "tube.db", default_chromecast="Xylo and Orchestra")
     process_playqueue(args, construct_tube_query)
 
 
-def printer(args):
+def printer(args) -> None:
     query = "select distinct ie_key, title, path from playlists"
     if "a" in args.print:
         query = f"""select
-            playlists.ie_key
+            playlists.is_deleted
+            , playlists.ie_key
             , playlists.title
+            , playlists.category
+            , playlists.profile download_profile
             , coalesce(playlists.path, "Playlist-less videos") path
             , sum(media.duration) duration
             , sum(media.size) size
             , count(*) count
         from media
         left join playlists on playlists.path = media.playlist_path
-        group by coalesce(playlists.path, "Playlist-less videos")"""
+        group by coalesce(playlists.path, "Playlist-less videos")
+        order by playlists.is_deleted desc, category, profile, playlists.path"""
 
     db_resp = list(args.db.query(query))
 
@@ -170,7 +119,7 @@ def printer(args):
 
         print(tabulate(tbl, tablefmt="fancy_grid", headers="keys", showindex=False))  # type: ignore
 
-        print(f"{len(db_resp)} playlists" if len(db_resp) > 1 else "1 playlist")
+        print(f"{len(db_resp)} playlists" if len(db_resp) >= 2 else "1 playlist")
         duration = sum(map(lambda m: m.get("duration") or 0, db_resp))
         if duration > 0:
             duration = human_time(duration)
@@ -178,7 +127,7 @@ def printer(args):
                 print("Total duration:", duration)
 
 
-def tube_list():
+def tube_list() -> None:
     parser = argparse.ArgumentParser(
         prog="library tubelist",
         usage="""library tubelist [database] [--print {p,f,a}] [--delete ...]
@@ -220,15 +169,12 @@ def tube_list():
     parser.add_argument("--print", "-p", nargs="*", default="p", choices=["p", "f", "a"], help=argparse.SUPPRESS)
     parser.add_argument("--delete", "--remove", "--erase", "--rm", "-rm", nargs="+", help=argparse.SUPPRESS)
     parser.add_argument("-v", "--verbose", action="count", default=0)
-    args = parser.parse_args()
-    log.info(dict_filter_bool(args.__dict__))
-
     args = parser.parse_args()
 
     if args.db:
         args.database = args.db
-
     args.db = db.connect(args)
+    log.info(utils.dict_filter_bool(args.__dict__))
 
     if args.delete:
         return delete_playlists(args, args.delete)
